'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

module.exports = function () {
    var axios = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var globalParams = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    return function (store) {
        return function (next) {
            return function (action) {

                if (!action || !action.axios || !action.axios.base) return next(action);

                var dispatch = store.dispatch;
                var params = action.axios;
                var basePoint = params.base;

                var axiosInstance = axios[basePoint];
                if (!axiosInstance) return next(action);

                var pointConfig = globalParams[basePoint] || {};

                var entity = params.entity || pointConfig.entity || globalParams.entity || 'entity';
                var url = params.url || pointConfig.url || globalParams.url || '/';
                var method = params.method || pointConfig.method || globalParams.method || 'get';
                var payloadStyle = params.payloadStyle || pointConfig.payloadStyle || globalParams.payloadStyle || 'plain';
                var suffixes = params.suffixes || pointConfig.suffixes || globalParams.suffixes || ['REQUEST', ' SUCCESS', 'FAILURE'];

                var defaultReqConfig = {
                    url: url,
                    method: method
                };

                var customReqConfig = params.config || {};
                var requestConfig = _extends({}, customReqConfig, defaultReqConfig);

                var typeRequest = entity.toUpperCase() + '_' + suffixes[0];
                var typeSuccess = entity.toUpperCase() + '_' + suffixes[1];
                var typeFailure = entity.toUpperCase() + '_' + suffixes[2];

                var isRequest = '' + suffixes[0].toLowerCase();
                var isSuccess = '' + suffixes[1].toLowerCase();
                var isFailure = '' + suffixes[2].toLowerCase();

                var requestAction = params.requestAction;

                if (params.type && params.payload) {
                    dispatch(params);
                } else if (requestAction && requestAction.type && requestAction.payload) {
                    dispatch(requestAction);
                } else if (payloadStyle === 'plain') {
                    dispatch({
                        type: typeRequest,
                        payload: {
                            status: isRequest
                        }
                    });
                } else if (payloadStyle === 'complex') {
                    var _payload;

                    dispatch({
                        type: typeRequest,
                        payload: (_payload = {}, _defineProperty(_payload, isRequest, true), _defineProperty(_payload, isSuccess, false), _defineProperty(_payload, isFailure, false), _payload)
                    });
                }

                var responseHandler = params.onResponse;
                var successHandler = params.onSuccess;
                var isAnyResponseHendlers = successHandler || responseHandler || false;
                var failureHandler = params.onFailure;

                var successAction = params.successAction;
                var validSuccessAction = false;
                if (successAction && successAction.type && successAction.payload && _typeof(successAction.payload) === 'object') validSuccessAction = true;

                var failureAction = params.failureAction;
                var validFailureAction = false;
                if (failureAction && failureAction.type && failureAction.payload && _typeof(failureAction.payload) === 'object') validFailureAction = true;

                axiosInstance(requestConfig).then(function (response) {
                    if (responseHandler) dispatch(responseHandler(response));
                    if (successHandler && !responseHandler) dispatch(successHandler(response.data));

                    if (validSuccessAction && !isAnyResponseHendlers) {
                        if (_typeof(successAction.payload) === 'object') {
                            successAction.payload = _extends({}, successAction.payload, { 'data': response.data });
                            dispatch(successAction);
                        } else {
                            console.log('The payload fild must be object');
                        }
                    } else if (!isAnyResponseHendlers && payloadStyle === 'plain') {
                        dispatch({
                            type: typeSuccess,
                            payload: {
                                status: isSuccess,
                                data: response.data
                            }
                        });
                    } else if (!isAnyResponseHendlers && payloadStyle === 'complex') {
                        var _payload2;

                        dispatch({
                            type: typeSuccess,
                            payload: (_payload2 = {}, _defineProperty(_payload2, isRequest, false), _defineProperty(_payload2, isSuccess, true), _defineProperty(_payload2, isFailure, false), _defineProperty(_payload2, 'data', response.data), _payload2)
                        });
                    }
                }).catch(function (error) {
                    if (failureHandler) dispatch(failureHandler(error));
                    if (validFailureAction && !failureHandler) {
                        if (_typeof(failureAction.payload) === 'object') {
                            failureAction.payload = _extends({}, failureAction.payload, { 'error': error });
                            dispatch(failureAction);
                        } else {
                            console.log('The payload fild must be object');
                        }
                    } else if (!failureHandler && payloadStyle === 'plain') {
                        dispatch({
                            type: typeFailure,
                            payload: {
                                status: isFailure,
                                error: error
                            }
                        });
                    } else if (!failureHandler && payloadStyle === 'complex') {
                        var _payload3;

                        dispatch({
                            type: typeFailure,
                            payload: (_payload3 = {}, _defineProperty(_payload3, isRequest, false), _defineProperty(_payload3, isSuccess, false), _defineProperty(_payload3, isFailure, true), _defineProperty(_payload3, 'error', error), _payload3)
                        });
                    }
                });
            };
        };
    };
};